SPARQLprov (PVLDB 2021, presented at VLDB 2022)

1. Introduction
---------------

This archive contains the material needed to reproduce the experiments
of the paper Computing How-Provenance for SPARQL Queries via Query
Rewriting, to be presented in VLDB 2022.  This document describes how
to use this material.

The paper introduces SPARQLprov, a query rewriting layer that computes
how-provenance for the query results provided by a SPARQL interface
(backend).  Conceptually, SPARQLprov runs in three steps:
1. query rewriting
2. execution of the rewritten query on the host engine
3. decoding of the answer into how- provenance polynomials.
Since step 2 dominates the overall runtime, this phase is the focus of
our runtime analysis.

Our experiments evaluate SPARQLprov on two backends:

- Fuseki (version 3.17.0)
  https://jena.apache.org/documentation/fuseki2

- Virtuoso Opensource Edition (version v7.2.5.1)
  https://github.com/openlink/virtuoso-opensource

These backends need to be configured for the actual hardware where the
experiments are run.  In our experiments we use the configuration
files that are in the folder `config`.  This configuration is defined
for a machine with 256 GB of RAM.  To run these experiments in a
different hardware, these parameters have to be modified.  The process
to set up the backends assumes the existence of these configuration
files.

We compared SPARQLprov against the following systems:

- GProM

  Arab, B. S., Feng, S., Glavic, B., Lee, S., Niu, X., & Zeng,
  Q. (2018). GProM-a swiss army knife for your provenance needs. A
  Quarterly bulletin of the Computer Society of the IEEE Technical
  Committee on Data Engineering, 41(1).

  Source code:
  https://github.com/IITDBGroup/gprom
  (master branch and commit #3a060fba)

  Using PostreSQL version 12
  https://www.postgresql.org

  See Section 2.3 on instructions to setup up GProM

- TripleProv

  Wylot, M., Cudre-Mauroux, P., & Groth, P. (2014, April). Tripleprov:
  Efficient processing of lineage queries in a native rdf store. In
  Proceedings of the 23rd international conference on World wide web
  (pp. 455-466).

  Source code:
  https://github.com/MarcinWylot/tripleprov_demo
  (master branch and commit #fa424a1f)

The evaluation was conducted on the following benchmarks and datasets:

- WatDiv

  G. Aluç, O. Hartig, M. T. Özsu and K. Daudjee. Diversified Stress
  Testing of RDF Data Management Systems. In Proc. The Semantic Web -
  ISWC 2014 - 13th International Semantic Web Conference, 2014, pages
  197-212.

  http://dsg.uwaterloo.ca/watdiv/.

- TPC-H

  The TPC-H is a decision support benchmark that consists of a suite
  of business oriented ad-hoc queries.  It is defined by the
  Transaction Processing Performance Council (TPC).

  http://www.tpc.org/tpch/.

- Wikidata benchmark

  This is a benchmark based on the Wikidata dataset defined in this
  paper and based on the benchmark presented in:

  Daniel Hernández, Aidan Hogan, Cristian Riveros, Carlos Rojas and
  Enzo Zerega. Querying Wikidata: Comparing SPARQL, Relational and
  Graph Databases. International Semantic Web Conference (ISWC), Kobe,
  Japan, Oct 17-21, 2016.

  The dataset used in our benchmark can be obtained from:
  https://zenodo.org/record/3746598

Our publication features the following experiments:

Experiment 1: Watdiv query execution times per reification scheme,
backend, and query template.  This experiment consists of:

- three reification schemes: named graphs, standard, and Wikidata.
- SPARQLprov with 2 different backends: Fuseki and Virtuoso.
- WatDiv benchmark with 25 query templates, 20 from the original
  WatDiv benchmark, and 5 additional query templates added to test
  SPARQLprov with the non-monotonic operator OPTIONAL. Each query
  template is instantiated to produce 10 queries, for which we execute
  3 variants:
  - P (the provenance query created by our rewriting approach on
    the respective reification scheme).
  - R (the query without provenance rewriting on the respective
    reification scheme).
  - B (the original query executed on the original (non-reified)
    WatDiv data.


Experiment 2: TPC-H query execution times per backend, size of the
dataset, and query template.  This experiment consists of:

- SPARQLprov with 2 different backends: Fuseki and Virtuoso.
- The TPC-H dataset with 9 scale factors.
- TPC-H benchmark with 11 select query templates.
- Each query template is instantiated into 10 concrete queries for
  which we execute 4 versions:
  - Base aggregate query (the TPC-H query translated to SPARQL; this
    is required since TPC-H includes SQL queries).
  - Provenance aggregate query (the provenance rewritten query for the
    base query).
  - Base non-aggregate query (the base query without aggregation).
  - Provenance non aggregate query (the provenance rewritten query for
    the base non-aggregate query).

Experiment 3: Querying a real dataset, Wikidata, which uses reification.

- SPARQLprov with 2 different backends: Fuseki and Virtuoso.
- Three groups of queries: star, minus, and union. We execute the
  following variants for these queries:
  - P (the provenance query created by our rewriting approach using
    the Wikidata reification scheme).
  - R (the query without provenance rewriting using the Wikidata
    reification scheme).
  - B (the original query executed on the non-reified Wikidata
    statements).

Experiment 4: Comparison of SPARQLprov against other solutions using
the WatDiv benchmark queries.

- SPARQLprov with Virtuoso as backend, and named graphs as reification
  scheme.
- GProM with PostgreSQL as backend, and vertical partitioning as
  schema.
- TripleProv using named graphs as reification scheme (the only
  reification scheme supported by TripleProv).


2. Installation
---------------

2.1 Setting up the environment
------------------------------

In this section we describe the basic environment needed to run the
experiments described in our paper.  The following are the basic
sofware that is needed:

- An Ubuntu 20.04 system with the following system packages listed in
  the file `system-packages.txt`.

- Ruby 3.0.0 (it can be installed using `rbenv`
  (https://github.com/rbenv/rbenv) and `ruby-build`
  (https://github.com/rbenv/ruby-build).

- Several Ruby libraries (ruby gems) that are automatically installed
  wit Bundler (https://bundler.io/) by running `bundle install`.

- The Boost library headers version 1.77.0 in the directory
  `/usr/local/boost_1_76_0`.  The instrunctions to install Boost can
  be found in:
  https://www.boost.org/

- The LXD containers system (https://linuxcontainers.org/lxd/).  This
  is installed as a snap package by running:

  ```
  sudo snap install lxd --channel=4.0/stable`
  ```

  and then following the initial configuration described in:
  https://linuxcontainers.org/lxd/getting-started-cli/#initial-configuration
  We use LXD containers to install the aforementioned backends and
  database systems in sandboxed environments.  LXD is a hypervisor to
  run both system containers (with LXC) and virtual machines (with
  QEMU) on Linux distributions.

The rest of the experiment is performed mostly automatically on top of
the Rake system, which is a Ruby DSL and system to automate processes
based on modular units called tasks.  Having completed the
instructions above, you have Rake installed in your system.  You can
display the complete list of tasks required to repeat our experiment
by running `rake --tasks`.  Then, you can run any of the listed tasks.
For instance, running `rake build_sparql_prov` in the main folder of
this project compiles SPARQLprov.  All tasks are run in the main
folder.

The database systems used by our experiments are automatically
compiled and installed in LXD containers via a set of Rake tasks.
That said, the configuration set for the systems may be not suitable
for rerunning our experiments in a different hardware.  The main issue
is the amount of memory reserved to the database process and query
executions. Users can change these configurations by editing the
following files:

- `config`
  - `fuseki`
    `config.ttl`
    `config-union-graph.ttl`
  - `virtuoso`
    `virtuoso.ini`
- `gprom`
  - `config`
    `postgresql.conf`

The process to build the containers assumes this location of the
configuration files, so they have to be modified before
building the containers.

2.2 Setting up SPARQLprov
-------------------------

SPARQLprov has two components: SPARQLprov-rewriting and
SPARQLprov-decoding, which correspond to the query rewriting and output
decoding phases.

The query rewriting component of SPARQLprov is implemented as a C++
library, which can be compiled by running `rake build_sparql_prov`.
The result of this compilation is the executable
`SPARQLprov/build/rewrite`.  See the compilation details in the file
`SPARQLprov/CMakeLists.txt`.

The decoding is done using the Ruby executable
`SPARQLprov/ruby/bin/decode_polynomials.rb`.

The rewritten query needs to be executed against a SPARQL endpoint.
SPARQL endpoints, as Virtuoso, and Fuseki are installed in LXD
containers as is described in Section 3.

2.3 Setting up GProM
--------------------

We use PostgreSQL as backend for GProM.  GProM and PostgreSQL are
installed automatically for the container of each dataset.  As
indicated in Section 2.1, after creating each GProM container, the
configuration file `gprom/config/postgresql.conf` is copied to the
container.  Hence, users must ensure this configuration file
is suitable for the hardware used to execute the experiments.

2.4 Setting up TripleProv
-------------------------

The code in folder `watdiv/tripleprov` is a copy of the original
TripleProv code, extended with the evaluation of the WatDiv queries.
The compilation of TripleProv is done by running
`rake build_tripleprov`.


3. Reproducing the results of Experiment 1
-----------------------------------------

The reproduction of Experiment 1 requires the following steps:

1. Downloading the base WatDiv dataset. This can be done by running
   the Rake task `watdiv_get_dataset`.  The downloaded file is saved
   in the directory `watdiv/datasets`.

2. Encoding the original dataset with different reification schemes
   via the Rake tasks `watdiv_generate_dataset_namedgraphs`,
   `watdiv_generate_dataset_standard`, and
   `watdiv_generate_dataset_wikidata`.  The new datasets are saved in
   the folder `watdiv/datasets`.

3. Building the container images for Fuseki and Virtuoso by running
   the Rake tasks `image_fuseki`, `image_fuseki_union_graph`, and
   `image_virtuoso`.  There are two images for Fuseki because we
   define the default graph for the named graph reification scheme as
   the union of all named graphs.  This is not needed for Virtuoso,
   because this engine does not include a separate default graph.

4. Loading the aforementioned datasets into Fuseki and Virtuoso
   via the following Rake tasks:

   - `create_watdiv_namedgraphs_fuseki`
   - `create_watdiv_standard_fuseki`
   - `create_watdiv_wikidata_fuseki`
   - `create_watdiv_namedgraphs_virtuoso`
   - `create_watdiv_standard_virtuoso`
   - `create_watdiv_wikidata_virtuoso`

   Each of these tasks create an LXD container with Fuseki or
   Virtuoso, and then loads the dataset into the corresponding
   database system.

5. Generating the benchmark queries.  The original WatDiv queries are
   stored in the directory `watdiv/queries/templates/B`.  Instead of
   using template variables, these queries use constants that can be
   replaced by other constants, called query parameters.  Query
   parameters are stored in the files in the directory `query_params`
   whereas the names of the constants that have to replaced are in the
   dictionary `QUERY_TEMPLATE_PARAMS_SCHEMES` in the Rake library file
   `generate_queries.rake`.

   In this experiment we use three different reification schemes: the
   named graphs reification scheme, the standard reification scheme,
   and the Wikidata reification scheme.  The base query templates are
   manually rewritten for these three reification schemes and saved in
   the folder `watdiv/queries/template`.

   The afforementioned queries are not the queries of the benchmark,
   but define the templates to generate them.  The benchmark queries
   are generated with the Rake task `watdiv_base_queries` and saved in
   the folder `watdiv/queries/100M`.  For each WatDiv query template
   `X` the following folders are generated inside the aforementioned
   folder:

   - `X/namedgraphs/B`
   - `X/namedgraphs/R`
   - `X/standard/B`
   - `X/standard/R`
   - `X/wikidata/B`
   - `X/wikidata/R`

   Each of these folders contain the queries for template `X`, a given
   reification scheme (namedgraphs, standard, or wikidata), and a
   given version of the query (B or R).

6. Running `SPARQLprov-rewriting` to rewrite all WatDiv queries (B) as
   provenance queries (P).  To rewrite all queries of the experiment
   we run the Rake task `watdiv_rewritten_queries`.  For each query
   template `X`, reification scheme `S`, and query `Q` in the folder
   `X/S/B`, this task generates a rewritten query in the folder
   `X/S/P`.  Hence, this Rake task generates queries in the following
   folders:

   - `X/namedgraphs/P`
   - `X/standard/P`
   - `X/wikidata/P`

7. Executing all the queries of this experiment by running the Rake
   task `experiment_one_bench`.  The result of this Rake task stores
   the answers in the folder `watdiv/answers`, and the execution time
   of the queries in the folder `watdiv/results`.  The answers are the
   body of the HTTP responses, whereas the results are CSV files with
   the following attributes:

   - `engine`: fuseki or virtuoso
   - `size`: 100M
   - `template`: the name of the template (e.g., C1, L3)
   - `scheme`: namedgraphs, standard, or wikidata
   - `mode`: B, R, or P
   - `query_id`: the ID of the query (remember that a query template
     is instantiated into several concrete queries)
   - `repetition`: 1 to 5 (recall that we execute each query 5 times
     and report averages across those executions)
   - `runtime`: the measured query runtime
   - `status`: the HTTP status (200 when success)

   The aforementioned Rake task `experiment_one_bench` starts a
   separate task for each pair defined by a query template, a
   reification scheme, and a mode.  Each of these tasks starts a
   container, starts the backend in the container, runs all the
   queries 6 times (the first time is used as warm-up and to capture
   the answers), and finally stops the backend and the container.

8. Analysis of the data and the figures in the paper are generated
   with the R files in folder `watdiv/data_analysis`.


4. Reproducing the results of Experiment 2
-----------------------------------------

This experiment assesses the scalability of the SPARQLprov using
queries in the TPC-H benchmarks.  To do so, we record runtimes for 9
TPC-H datasets using different scale factors ranging from 0.01 to 1.

The reproduction of Experiment 2 requires the following steps:

1. Generating the TPC-H dataset using the `dbgen` tool (not included
   here due to licensing restrictions).  The TPC-H tools can be
   manually downloaded from the official website
   (http://www.tpc.org/tpch/).  To integrate this tool into the
   pipeline you have to copy the uncompressed TPC-H file in
   `tpch/tpch-tool/tpc-h-tool.zip` (we use the version `2.18.0_rc2`).
   Then, you can compile the TPC-H tool with the Rake task
   `build_tpch_tool`.  This task automatically sets the parameters of
   the Makefile used for compilation.  These parameters are copied
   from the file `tpch/tpch-tool/Makefile`.

2. After compiling the `dbgen` you can use it to generate the datasets
   for each of the 9 scale factors we consider in this experiment.
   This is done with the Rake task `generate_tpch_tbl_datasets`.  This
   task generates several folders with prefix `ds_tbl_` inside the
   folder `tpch/datasets`.  Each of these folders contains the TBL
   files for the TPC-H dataset of a specific scale factor.

3. The datasets generated in the previous step need to be translated
   to RDF and loaded into an RDF backend to be queried by SPARQLprov.
   The translation from TBL to TTL is done with the Rake task
   `generate_tpch_ttl_datasets`.  This task generates several folders
   with prefix `ds_ttl_` inside the folder `tpch/datasets`.  Each of
   these folders contains the RDF/Turtle files for the TPC-H dataset
   of a specific scale factor.

4. Creating the container images for Fuseki and Virtuoso containers by
   running the Rake tasks `image_fuseki` and `image_virtuoso`.

5. Creating a container for each of the aforementioned datasets and
   backends (namely Virtuoso and Fuseki).  These containers are
   created with the Rake task `create_tpch`.

6. The TPC-H benchmark is defined by replacing some parameters in
   query templates that are in the folder `tpch/sparql_examples`.
   These SPARQL templates are generated manually by translating the
   SQL query templates of the TPC-H benchmarks.  The parameters values
   to generate queries for these templates are generated by running
   the Rake task `tpch_parameters`.  This task calls the `qgen`
   executable of the TPC-H tool to produce random parameters values
   for the TPC-H queries.  The `qgen` executable was already compiled
   with the task `build_tpch_tool` in step 1.  The generated parameter
   values are saved in the folder `tpch/query_parameters`.

7. Using the parameters to build concrete queries with the Rake task
   `tpch_queries`.  This Rake task saves a query for each instance of
   the template into the folders `tpch/queries/X/base`,
   `tpch/queries/X/base_non_aggregate`, `tpch/queries/X/prov`, and
   `tpch/queries/X/prov_non_aggregate`, where `X` is the name of a
   query template.  In this particular case the query rewriting cannot
   be applied on the base query, because it does not express the
   information about the original relationships that are being
   reified.  This is because the reification scheme is not based on
   relationships codified by triples but on n-ary relationships.

8. Executing all the queries of this experiment by running the Rake
   task `experiment_two_bench`.  The result of this task stores the
   answers in the folder `tpch/answers`, and the execution time of the
   queries in the folder `tpch/results`.  The answers are the body of
   the HTTP responses, whereas the results are CSV files with the
   following attributes:

   - `engine`: fuseki or virtuoso
   - `scale_factor`: the scale factor used to create the dataset
     (0.01, 0.0178, 0.0316, 0.0573, 0.1, 0.178, 0.316, 0.573, or 1).
   - `template`: the name of the template (01, 03, 05, 06, 07, 08, 09,
     10, 12, 14, or 19).
   - `mode`: base, base_non_aggregate, prov, or prov_non_aggregate.
   - `query_id`: the ID of the query (recall that a template has
     several query instances)
   - `repetition`: 1 to 5 (recall that we execute each query 5 times
     and report averages across those executions)
   - `runtime`: the measured query runtime
   - `status`: the HTTP status (200 when success)

   The aforementioned Rake task `experiment_two_bench` starts a
   separate task for each pair defined by a query template, scale
   factor, and mode.  Each of these tasks starts a container, starts
   the backend in the container, runs all the queries 6 times (the
   first time is used as warm-up and to capture the answers), and
   finally stops the backend and the container.

9. The analysis of the data and the figures in the paper are generated
   with the R files in folder `tpch/data_analysis`.


5 Reproducing the results of Experiment 3
-----------------------------------------

The reproduction of Experiment 3 requires the following steps:

1. Downloading the Wikidata dataset via the Rake task
   `wikidata_get_dataset`.  The dataset is downloaded from
   https://zenodo.org/record/3746598 and saved in the directory
   `watdiv/dataset`.

2. Creating the container images for Fuseki and Virtuoso by running
   the Rake tasks `image_fuseki` and `image_virtuoso`.

3. Loading the Wikidata dataset into Fuseki and Virtuoso via the Rake
   tasks:

   - `create_wikidata_fuseki`
   - `create_wikidata_virtuoso`

   Each of these tasks creates an LXD container with Fuseki and
   Virtuoso, and then loads the dataset into the corresponding
   database system.

4. Generating the benchmark queries. The Wikidata benchmark queries
   are of three types: B (base), R (using reification), and P
   (provenance rewritten queries).  Queries of types B and P reside
   already in the folder `wikidata/queries` whereas the queries of
   type P have to be generated with by running the Rake task
   `wikidata_rewritten_queries`.  The final structure for the folder
   `wikidata/queries` is the following:

   - `minus`
     - `B`
     - `R`
     - `P`
    - `star`
     - `B`
     - `R`
     - `P`
    - `union`
     - `B`
     - `R`
     - `P`

   The folders `minus/P`, `star/P`, and `union/P` store the queries
   that are generated using the query rewriting.

4. Executing all the queries of this experiment by running the Rake
   task `experiment_three_bench`.  The result of this task stores the
   answers in the folder `wikidata/answers` and the execution time of
   the queries in the folder `wikidata/results`.  The answers are the
   body of the HTTP responses, whereas the results are CSV files with
   the following attributes:

   - `engine`: fuseki or virtuoso
   - `template`: minus, star, or union
   - `mode`: B, R, or P
   - `query_id`: the ID of the query (a template have several query instances)
   - `repetition`: 1 to 5 (recall that we execute each query 5 times
     and report averages across those executions)
   - `runtime`: the measured query runtime
   - `status`: the HTTP status (200 when success)

   The aforementioned Rake task `experiment_three_bench` starts a
   separate task for each pair defined by a query template and a mode.
   Each of these tasks starts a container, starts the backend in the
   container, runs all the queries 6 times (the first time is used as
   warm-up and to capture the answers), and finally stops the backend
   and the container.

7. The analysis of the data and the figures in the paper are generated
   with the R files in folder `wikidata/data_analysis`.


6. Reproducing the results of Experiment 4
-----------------------------------------

The reproduction of Experiment 4 requires the following steps:

1. Downloading the base WatDiv dataset with the Rake task
   `watdiv_get_dataset`.  The downloaded file is saved in the
   directory `watdiv/datasets`.

2. Translating the original dataset into the named graph reification
   scheme with the Rake task `watdiv_generate_dataset_namedgraphs`.
   The generated dataset dataset is saved in the folder
   `watdiv/datasets`.

3. Translating the original dataset to a CSV file (a table to be
   queried using SQL) with the Rake task
   `watdiv_generate_dataset_table`.

4. Creating the container images for Virtuoso and GProM by running the
   Rake task `image_virtuoso` and `image_gprom`.

5. Loading the aforementioned datasets into Virtuoso via the Rake task
   `create_watdiv_namedgraphs_virtuoso`.  For virtuoso, launch a LXD
   container named `watdiv-gprom-postgresql` using the image
   `gprom-postgres`.  Then load the CSV data generated in the step 3
   into the PostgreSQL, creating a table with the following schema:

   ```
   CREATE TABLE quads (quad_predicate text, quad_subject text, quad_object text);
   CREATE INDEX quads_idx_predicate ON quads (quad_predicate);
   ```

   Then run the script `watdiv/bin/generate_vp.rb` in the main folder.
   This script translates the aforementioned data to the vertical
   partitioning scheme.

6. Generating the benchmark queries. The original WatDiv queries are
   stored in the directory `watdiv/queries/templates/B`.  Instead of
   using template variables, these queries use constants that can be
   replaced by other constants, called query parameters.  Query
   parameters are stored in the files in the directory `query_params`,
   whereas the names of the constants that have to replaced are in the
   dictionary `QUERY_TEMPLATE_PARAMS_SCHEMES` in the Rake library file
   `generate_queries.rake`.

   In this experiment we use the named graphs reification scheme.
   The base query templates are manually rewritten and saved in the
   folder `watdiv/queries/template/namedgraphs`.

   Watdiv offers query templates, therefore the actual benchmark
   queries can be generated with the Rake task `watdiv_base_queries`
   that saves them in the folder `watdiv/queries/100M`.  For each
   WatDiv query template `X` the following folders are generated
   inside the aforementioned folder:

   - `X/namedgraphs/B`
   - `X/namedgraphs/R`

   Each of these folders contain the queries for template `X` and a
   given version of the query (B or R).

7. To allow for comparison with GProM, we manually create an SQL query
   template for each of the SPARQL query templates of the WatDiv
   experiments (this does not include the additional OPTIONAL queries
   in Experiment 1).  For each of these SQL query templates we create
   query instances replacing the template parameters with the Rake
   task `watdiv_sql_queries`.  This generates queries in the folder
   `watdiv/queries/100M/sql/P`.

8. We then run `SPARQLprov-rewriting` to rewrite all WatDiv queries
   (B) as provenance queries (P).  To rewrite all queries we run the
   Rake task `watdiv_rewritten_queries`.  For each query template `X`
   and query `Q` in the folder `X/namedgraphs/B`, this task generates
   a rewritten query in the folder `X/namedgraphs/P`.  Hence, this
   Rake task generates queries in the following folders:

   - `X/namedgraphs/P`

9. Executing all the queries of this experiment by running the Rake
   task `experiment_four_bench`.  The result of this task stores the
   answers in the folder `watdiv/answers` and the execution time of
   the queries in the folder `watdiv/results`.  The answers are the
   body of the HTTP responses, whereas the results are stored in CSV
   files with the following attributes:

   - `engine`: virtuoso or gprom
   - `size`: 100M
   - `template`: the name of the template (e.g., C1, L3)
   - `scheme`: namedgraphs (for virtuoso), or vp (for the vertical
     partitioning scheme used for GProM)
   - `mode`: P
   - `query_id`: the ID of the query (recall that a template is
     instantiated into several query instances)
   - `repetition`: 1 to 5 (recall that we execute each query 5 times
     and report averages across those executions)
   - `runtime`: the measured query runtime
   - `status`: the HTTP status (200 when success)

   The aforementioned task `experiment_four_bench` starts a separate
   task for each pair defined by a query template, a reification
   scheme, and a mode.  Each of these tasks starts a container, starts
   the backend in the container, runs all the queries 6 times (the
   first time is used as warm-up and to capture the answers), and
   finally stops the backend and the container.

10. Executing the benchmark on TripleProv. To do so, we first create
  the directory `watdiv/tripleprov/watdiv_dataset` and copy the named
  graphs version of the WatDiv dataset to the created directory.  The
  files have to be decompressed, named with the suffix `nt`, and you
  have to remove all instances of the char `~` from the data.  You can
  do the latter step using the `sed` command line utility.

11. After preparing the data for TripleProv (step 10), run the
  benchmark with the Rake task `experiment_four_tripleprov`.  The
  answers are written in the file `watdiv/answers/tripleprov.txt`.
  The runtimes codified on this file have to be copied manually to a
  CSV file with the following structure:

   - `engine`: tripleprov
   - `size`: 100M
   - `template`: the name of the template (e.g., C1, L3)
   - `scheme`: namedgraphs
   - `mode`: T
   - `query_id`: the ID of the query
   - `repetition`: 1 to 5
   - `runtime`: the runtime
   - `status`: 200

12. The analysis of the data and the figures in the paper are generated
    with the R files in folder `watdiv/data_analysis`.


7. How to decode the polynomials from the rewritten query answers
----------------------------------------------------------------

To display the polynomials codified in the answers of the rewritten
queries, you can use our decoding tool on the outputs of the benchmark
task.  An example of such output is the file
`query_answers/watdiv/L3.csv` that contains a subset of the mappings
returned by the rewritten version of query L3, and encodes a
provenance polynomial for each answer.  The following command shows
how this result is displayed:

```
ruby/bin/decode_polynomials.rb ../query_answers/watdiv/L3.csv
{:v0=>"http://db.uwaterloo.ca/~galuc/wsdbm/User43337"} =>
  ( + ( * ( + <http://example.org/statement/53412926> )
          ( + <http://example.org/statement/53412932> ) ) )
{:v0=>"http://db.uwaterloo.ca/~galuc/wsdbm/User50045"} =>
  ( + ( * ( + <http://example.org/statement/60019210> )
          ( + <http://example.org/statement/60019225> ) )
      ( * ( + <http://example.org/statement/60019211> )
          ( + <http://example.org/statement/60019225> ) ) )
```
