#!/usr/bin/env ruby
# frozen_string_literal: true

require 'csv'
require 'sparql'
require 'sparql/client'
require 'pry'
require_relative '../lib/polynomial_table'

# ruby/bin/lineage_subgraph.rb query_answers/watdiv/L3.csv http://10.246.87.209:3030/ds/sparql

if ARGV.size != 2
  puts 'Usage: lineage_subgraph.rb <query_answers> <endpoint> <reification_scheme>'
  exit 1
end

CSV_TABLE = CSV.table(ARGV[0])
ENDPOINT = ARGV[1]

k_relation = SPARQLprov.decode(CSV_TABLE)
statements = k_relation.map { |_, polynomial| polynomial.lineage }.inject(&:+)

sparql = SPARQL::Client.new(ENDPOINT)

def wikidata_reified_triple(statement_uri)
  serialized_uri = RDF::NQuads.serialize(statement_uri)
  query_string = <<-EOS
  SELECT ?s ?p ?o
  WHERE {
    ?s ?p1 #{serialized_uri} .
    FILTER (regex(xsd:string(?p1), "^http://www.wikidata.org/prop/"))
    BIND(IRI(replace(xsd:string(?p1), "http://www.wikidata.org/prop/", "http://www.wikidata.org/prop/statement/")) AS ?p2)
    #{serialized_uri} ?p2 ?o .
    BIND(IRI(replace(xsd:string(?p1), "http://www.wikidata.org/prop/", "http://www.wikidata.org/prop/direct/")) AS ?p)
  }
  EOS
  SPARQL.parse(query_string)
end

def namedgraphs_reified_triple(statement_uri)
  sparql
    .select
    .graph(statement_uri)
    .where(%i[subject predicate object])
end

# The following only works for the named graphs reification scheme
statements.each do |statement|
  statement_uri = RDF::URI(statement)

  query = wikidata_reified_triple(statement_uri)

  serialized_uri = RDF::NQuads.serialize(statement_uri)
  query_string = <<-EOS
  SELECT ?subject ?predicate ?object ?p1 ?p2
  WHERE {
    ?subject ?p1 #{serialized_uri} .
    FILTER (regex(xsd:string(?p1), "^http://www.wikidata.org/prop/"))
    BIND(IRI(replace(xsd:string(?p1), "http://www.wikidata.org/prop/", "http://www.wikidata.org/prop/statement/")) AS ?p2)
    #{serialized_uri} ?p2 ?object .
    BIND(IRI(replace(xsd:string(?p1), "http://www.wikidata.org/prop/", "http://www.wikidata.org/prop/direct/")) AS ?predicate)
  }
  EOS

  # sparql.query(query_string).each_solution do |solution|
  #   puts "#{RDF::NQuads.serialize(solution[:subject])} " \
  #        "#{RDF::NQuads.serialize(solution[:predicate])} " \
  #        "#{RDF::NQuads.serialize(solution[:object])} " \
  #        "#{RDF::NQuads.serialize(statement_uri)} ."
  # end

  sparql.query(query_string).each_solution do |solution|
    puts "#{RDF::NQuads.serialize(solution[:subject])} " \
         "#{RDF::NQuads.serialize(solution[:predicate])} " \
         "#{RDF::NQuads.serialize(solution[:object])} .\n" \
         "#{RDF::NQuads.serialize(solution[:subject])} " \
         "#{RDF::NQuads.serialize(solution[:p1])} " \
         "#{RDF::NQuads.serialize(statement_uri)} .\n" \
         "#{RDF::NQuads.serialize(statement_uri)} " \
         "#{RDF::NQuads.serialize(solution[:p2])} " \
         "#{RDF::NQuads.serialize(solution[:object])} ."
  end
end.join(', ')
