#define BOOST_TEST_MODULE spm_polynomial
#include <iostream>
#include <boost/test/included/unit_test.hpp>
//#include <boost/test/included/unit_test.hpp>
#include "SPMPolynomial.hpp"

namespace utf = boost::unit_test;

BOOST_AUTO_TEST_CASE(zero_polynomial_creation,
                     * utf::description("Polynomial: Create constants"))
{
  std::ostringstream os;
  SPM::Polynomial<std::string> polynomial;
  os << polynomial;
  BOOST_TEST( os.str() == "0" );
}

BOOST_AUTO_TEST_CASE(constant_polynomial_creation,
                     * utf::description("Polynomial: Create constants"))
{
  std::ostringstream os;
  SPM::Polynomial<std::string> polynomial(3);
  os << polynomial;
  BOOST_TEST( os.str() == "3" );
}

BOOST_AUTO_TEST_CASE(term_polynomial_creation,
                     * utf::description("Polynomial: Create variables"))
{
  std::ostringstream os;
  SPM::Polynomial<std::string> polynomial("x");
  os << polynomial;
  BOOST_TEST( os.str() == "( variable x )" );
}

BOOST_AUTO_TEST_CASE(polynomial_sum_creation,
                     * utf::description("Polynomial: Polynomial operations"))
{
  std::ostringstream os;
  SPM::Polynomial<std::string> p1(1);
  SPM::Polynomial<std::string> p2(2);
  std::vector<SPM::Polynomial<std::string>> operands = { p1, p2 };
  SPM::Polynomial<std::string> polynomial(SPM::Operation::Sum, operands);
  os << polynomial;
  BOOST_TEST( os.str() == "( + 1 2 )" );
}

BOOST_AUTO_TEST_CASE(polynomial_product_creation,
                     * utf::description("Polynomial: Polynomial operations"))
{
  std::ostringstream os;
  SPM::Polynomial<std::string> p1(1);
  SPM::Polynomial<std::string> p2(2);
  std::vector<SPM::Polynomial<std::string>> operands = { p1, p2 };
  SPM::Polynomial<std::string> polynomial(SPM::Operation::Product, operands);
  os << polynomial;
  BOOST_TEST( os.str() == "( * 1 2 )" );
}

BOOST_AUTO_TEST_CASE(polynomial_difference_creation,
                     * utf::description("Polynomial: Polynomial operations"))
{
  std::ostringstream os;
  SPM::Polynomial<std::string> p1(1);
  SPM::Polynomial<std::string> p2(2);
  std::vector<SPM::Polynomial<std::string>> operands = { p1, p2 };
  SPM::Polynomial<std::string> polynomial(SPM::Operation::Difference, operands);
  os << polynomial;
  BOOST_TEST( os.str() == "( - 1 2 )" );
}

BOOST_AUTO_TEST_CASE(polynomial_sums,
                     * utf::description("Polynomial: Sums"))
{
  std::ostringstream os;
  SPM::Polynomial<std::string> p1("x");
  SPM::Polynomial<std::string> p2("y");
  os << p1 + p2;
  BOOST_TEST( os.str() == "( + ( variable x ) ( variable y ) )" );
}

BOOST_AUTO_TEST_CASE(polynomial_products,
                     * utf::description("Polynomial: Sums"))
{
  std::ostringstream os;
  SPM::Polynomial<std::string> p1("x");
  SPM::Polynomial<std::string> p2("y");
  os << p1 * p2;
  BOOST_TEST( os.str() == "( * ( variable x ) ( variable y ) )" );
}

BOOST_AUTO_TEST_CASE(polynomial_differences,
                     * utf::description("Polynomial: Sums"))
{
  std::ostringstream os;
  SPM::Polynomial<std::string> p1("x");
  SPM::Polynomial<std::string> p2("y");
  os << p1 - p2;
  BOOST_TEST( os.str() == "( - ( variable x ) ( variable y ) )" );
}
