# frozen_string_literal: true

desc 'Create GProM container for PostgreSQL'
named_task 'image_gprom_postgres' do
  container = 'gprom-postgres'

  puts "Lauching container #{container}"
  launch_ubuntu_container(container, '20.04')

  puts 'Install required system packages'
  sh "lxc exec #{container} -- apt update"
  sh "lxc exec #{container} -- apt -y upgrade"
  sh "lxc exec #{container} -- apt install -y " \
     'build-essential postgresql libpq-dev autotools-dev automake libtool ' \
     'bison flex unzip postgresql-server-dev-all pq'

  code_url = 'https://github.com/IITDBGroup/gprom/archive/refs/tags/master.zip'
  sh "lxc exec #{container} -- su - ubuntu -c 'wget #{code_url}'"
  sh "lxc exec #{container} -- su - ubuntu -c 'unzip master.zip'"
  sh "lxc exec #{container} -- su - ubuntu -c 'rm master.zip'"
  sh "lxc exec #{container} -- su - ubuntu -c " \
     "'cd gprom-master && ./autogen.sh && ./configure " \
     '--disable-oracle --disable-sqlite --disable-monetdb ' \
     '--disable-readline --disable-java --with-libpq=/usr/lib/x86_64-linux-gnu/libpq.so ' \
     "--with-libpq-headers=/usr/include/postgresql'"
  sh "lxc exec #{container} -- su - ubuntu -c 'cd gprom-master && make'"
  sh "lxc exec #{container} -- su - ubuntu -c 'cd gprom-master && sudo make install'"
  sh "lxc file push config/postgres/postgresql.conf #{container}/etc/postgresql/12/main/postgresql.conf"

  # Publish the container
  puts 'Create a container image'
  system "lxc stop #{container}"
  system "lxc publish #{container} --alias #{container}"
end

desc 'Create GProM container for PostgreSQL'
task image_gprom: task_dependency('image_gprom_postgres')
