# frozen_string_literal: true

def tpch_bench(endpoint, scale_factor, template, mode, times = 5)
  puts "Starting workload #{[endpoint, scale_factor, template, mode].join('-')}"

  endpoint.start
  queries = Dir[File.join('tpch/queries', template, mode, scale_factor, 'q*.sparql')].sort

  results = "tpch/results/#{endpoint.name}-#{scale_factor}-#{template}-#{mode}.csv"
  FileUtils.mkdir_p(File.dirname(results))

  # puts 'Checking if this query produces timeouts'
  # out = endpoint.bench_query(queries.first)
  # puts "result time=#{out[0]} status=#{out[1]}"
  # if out[0] >= endpoint.timeout or out[1] != '200'
  #   puts 'timeout detected'
  #   CSV.open(results, 'w') do |csv|
  #     csv << %w{engine scale_factor template mode query_id repetition time status}
  #     queries.each do |query|
  #       query_name = File.basename(query).sub(/.sparql$/, '')
  #       csv << [endpoint.name, scale_factor.sub('d', '.'), template, mode,
  #               query_name, 1, endpoint.timeout, 500]
  #     end
  #   end
  #   endpoint.stop
  #   return
  # else
  #   puts "no timeout"
  # end

  queries.each do |query|
    puts "warming up #{query}"
    answers = query
                .gsub('/', '-')
                .sub('.sparql', '.csv')
                .sub('tpch-queries-', "tpch/answers/#{endpoint.name}-")
    FileUtils.mkdir_p(File.dirname(answers))
    File.open(answers, 'w') do |file|
      file.write endpoint.run_query(query)
    end
  end

  CSV.open(results, 'w') do |csv|
    csv << %w{engine scale_factor template mode query_id repetition time status}
    (1..times).each do |repetition|
      queries.each do |query|
        puts "running query #{query} (repetition #{repetition})"
        out = endpoint.bench_query(query)
        query_name = File.basename(query).sub(/.sparql$/, '')
        csv << [endpoint.name, scale_factor.sub('d', '.'), template, mode,
                query_name, repetition, out[0], out[1]]
        csv.flush
      end
    end
  end

  endpoint.stop
end

tpch_bench_dependencies = []

%w{fuseki virtuoso}.each do |engine|
  Dir['tpch/datasets/ds_ttl_*'].sort.each do |dataset|
    scale_factor = dataset.gsub('tpch/datasets/ds_ttl_', '').gsub('.', 'd')
    Dir['tpch/queries/*'].sort.each do |query_dir|
      template = query_dir.sub('tpch/queries/', '')
      Dir[File.join(query_dir, '*')].each do |mode_dir|
        mode = mode_dir.sub("#{query_dir}/", '')
        task_name = "tpch_bench_#{scale_factor}_#{template}_#{mode}_#{engine}"

        desc "Run bench for scale factor=#{scale_factor} template=#{template}, mode=#{mode}, engine=#{engine}"
        named_task task_name do
          case engine
          when 'fuseki'
            endpoint = LXDFusekiEndpoint.new("tpch-#{scale_factor}-fuseki3-ubuntu2004")
          when 'virtuoso'
            endpoint = LXDVirtuosoEndpoint.new("tpch-#{scale_factor}-virtuoso7-debian9")
          end
          tpch_bench(endpoint, scale_factor, template, mode)
        end

        tpch_bench_dependencies.append(task_dependency(task_name))
      end
    end
  end
end

desc 'Run bench'
task experiment_two_bench: tpch_bench_dependencies
