# frozen_string_literal: true

require_relative '../../lib/load_virtuoso'

def tbl_dir(scale)
  "#{TPCH_TOOL_DATASET_DIR}/ds_tbl_#{scale}"
end

SCALES.each do |scale|
  desc "Create TBL TPCH dataset for scale #{scale}"
  file tbl_dir(scale) => TPCH_TOOL_DBGEN do
    sh "cd #{File.dirname TPCH_TOOL_DBGEN} && ./dbgen -s #{scale}"
    sh "mkdir -p #{tbl_dir(scale)}"
    sh "mv #{File.dirname TPCH_TOOL_DBGEN}/*.tbl #{tbl_dir(scale)}"
    sh "pigz #{tbl_dir(scale)}/*.tbl"
  end
end

desc 'Generate TPC-H TBL datasets'
task generate_tpch_tbl_datasets: SCALES.map { |scale| tbl_dir(scale) }

def ttl_dir(scale)
  "#{TPCH_TOOL_DATASET_DIR}/ds_ttl_#{scale}"
end

SCALES.each do |scale|
  desc "Create TTL TPCH dataset for scale #{scale}"
  file ttl_dir(scale), tbl_dir(scale) do
    sh "tpch/bin/tbl_to_rdf.rb #{tbl_dir(scale)}"
  end
end

desc 'Generate TPC-H TTL datasets'
task generate_tpch_ttl_datasets: SCALES.map { |scale| ttl_dir(scale) }

create_tpch_dependencies = []

%w[fuseki virtuoso].each do |engine|
  SCALES.each do |scale|
    task_description = "Create TPCH container for engine=#{engine} scale=#{scale}"
    task_name = "create_tpch_#{engine}_#{scale}"

    desc task_description
    named_task task_name do
      scale_factor = scale.sub('.', 'd')
      setting = {
        'fuseki' => {
          image: 'fuseki3-ubuntu2004',
          user: 'ubuntu'
        },
        'virtuoso' => {
          image: 'virtuoso7-debian9',
          user: 'debian'
        }
      }
      image = setting[engine][:image]
      user = setting[engine][:user]
      container = "tpch-#{scale_factor}-#{image}"
      sh "lxc launch #{image} #{container}"
      File["tpch/datasets/ds_ttl_#{scale}/*.ttl.gz"].each do |file|
        system "lxc file push #{file} #{container}/home/#{user}/dataset"
      end
      puts 'Loading data'
      if engine == 'fuseki'
        sh "lxc exec #{container} -- su - #{user} -c " \
           "'apache-jena-3.17.0/bin/tdbloader2 --loc tdb dataset/* > load-1.log 2> load-2.log'"
      else
        virtuoso_7_load_dataset(container)
      end
    end

    desc task_description
    task task_name

    create_tpch_dependencies << task_name
  end
end

desc 'Create TPC-H containers'
task create_tpch: create_tpch_dependencies
