# frozen_string_literal: true

WATDIV_DATASET_DIR = 'watdiv/datasets'
WATDIV_ORIGINAL_FILE = "#{WATDIV_DATASET_DIR}/watdiv.100M.nt.gz".freeze
WATDIV_NAMEDGRAPHS_FILE = "#{WATDIV_DATASET_DIR}/watdiv.100M-namedgraphs.nt.gz".freeze
WATDIV_RDF_FILE = "#{WATDIV_DATASET_DIR}/watdiv.100M-rdf.nt.gz".freeze
WATDIV_WIKIDATA_FILE = "#{WATDIV_DATASET_DIR}/watdiv.100M-wikidata.nt.gz".freeze
WATDIV_SPOG_FILE = "#{WATDIV_DATASET_DIR}/watdiv.100M-spog.csv.gz".freeze
WATDIV_PSOG_FILE = "#{WATDIV_DATASET_DIR}/watdiv.100M-psog.csv.gz".freeze

desc 'Get the original WatDiv dataset'
task watdiv_get_dataset: WATDIV_ORIGINAL_FILE

desc 'Get the original WatDiv dataset'
file WATDIV_ORIGINAL_FILE do
  dataset_url = 'https://dsg.uwaterloo.ca/watdiv/watdiv.10M.tar.bz2'
  sh "mkdir -p #{WATDIV_DATASET_DIR} && cd #{WATDIV_DATASET_DIR} && wget #{dataset_url}"
  downloaded_file = "#{WATDIV_DATASET_DIR}/watdiv.100M.tar.bz2".freeze
  sh "tar -xjf #{downloaded_file}"
  sh "rm #{downloaded_file} #{WATDIV_DATASET_DIR}/saved.txt"
  sh "pigz #{WATDIV_DATASET_DIR}/watdiv.100M.nt"
end

desc 'Generate WatDiv dataset for the namedgraphs reification scheme'
file WATDIV_NAMEDGRAPHS_FILE => WATDIV_ORIGINAL_FILE do
  sh "zcat #{WATDIV_ORIGINAL_FILE} | watdiv/bin/reify.rb namedgraphs | " \
     "pigz > #{WATDIV_NAMEDGRAPHS_FILE}"
end

desc 'Generate WatDiv dataset for the namedgraphs reification scheme'
task watdiv_generate_dataset_namedgraphs: WATDIV_NAMEDGRAPHS_FILE

desc 'Generate WatDiv dataset for standard reification scheme'
file WATDIV_RDF_FILE => WATDIV_ORIGINAL_FILE do
  sh "zcat #{WATDIV_ORIGINAL_FILE} | watdiv/bin/reify.rb rdf | " \
     "pigz > #{WATDIV_RDF_FILE}"
end

desc 'Generate WatDiv dataset for standard reification scheme'
task watdiv_generate_dataset_standard: WATDIV_RDF_FILE

desc 'Generate WatDiv dataset for the Wikidata reification scheme'
file WATDIV_WIKIDATA_FILE => WATDIV_ORIGINAL_FILE do
  sh "zcat #{WATDIV_ORIGINAL_FILE} | watdiv/bin/reify.rb wikidata | " \
     "pigz > #{WATDIV_WIKIDATA_FILE}"
end

desc 'Generate WatDiv dataset for Wikidata reification scheme'
task watdiv_generate_dataset_wikidata: WATDIV_WIKIDATA_FILE

desc 'Generate WatDiv dataset for PSO scheme'
file WATDIV_PSOG_FILE => WATDIV_ORIGINAL_FILE do
  sh "zcat #{WATDIV_ORIGINAL_FILE} | watdiv/bin/reify.rb pso | " \
     "pigz > #{WATDIV_PSOG_FILE}"
end

desc 'Generate WatDiv dataset for PSO scheme'
task watdiv_generate_dataset_table: WATDIV_PSOG_FILE

desc 'Generate WatDiv datasets'
task generate_watdiv_datasets:
       %i[
         watdiv_generate_dataset_namedgraphs
         watdiv_generate_dataset_standard
         watdiv_generate_dataset_wikidata
         watdiv_generate_dataset_table
       ]
