# frozen_string_literal: true

def watdiv_bench(endpoint, size, template, scheme, mode, times = 5)
  puts "Starting workload #{[endpoint, size, template, scheme, mode].join('-')}"

  endpoint.start
  queries = Dir[File.join('watdiv/queries', size, template, scheme, mode, '*.sparql')].sort

  queries.each do |query|
    puts "warming up #{query}"
    answers = query
                .gsub('/', '-')
                .sub('.sparql', '.csv')
                .sub('watdiv-queries-', "watdiv/answers/#{endpoint.name}-")
    FileUtils.mkdir_p(File.dirname(answers))
    File.open(answers, 'w') do |file|
      file.write endpoint.run_query(query)
    end
  end

  results = "watdiv/results/#{endpoint.name.sub('-', '')}-#{size}-#{template}-#{scheme}-#{mode}.csv"
  FileUtils.mkdir_p(File.dirname(results))
  CSV.open(results, 'w') do |csv|
    csv << %w[engine size template scheme mode query_id repetition time status]
    (1..times).each do |repetition|
      queries.each do |query|
        puts "running query #{query} (repetition #{repetition})"
        out = endpoint.bench_query(query)
        query_name = File.basename(query).sub(/.sparql$/, '')
        csv << [endpoint.name, size, template, scheme, mode, query_name, repetition, out[0], out[1]]
      end
    end
  end

  endpoint.stop
end

watdiv_bench_dependencies = []

%w[virtuoso fuseki].each do |engine|
  size = '100M'
  %w[C1 C2 C3 F1 F2 F3 F4 F5 L1 L2 L3 L4 L5 S1 S2 S3 S4 S5 S6 S7].each do |template|
    %w[namedgraphs rdf wikidata].each do |scheme|
      %w[B R P].each do |mode|
        task_name = "run_watdiv_bench_#{engine}_#{size}_#{template}_#{scheme}_#{mode}"
        desc "Run bench for #{engine} #{size} #{template} #{scheme} #{mode}"
        named_task task_name do
          case engine
          when 'fuseki'
            endpoint = LXDFusekiEndpoint.new("watdiv-#{size}-#{scheme}-fuseki3-ubuntu2004")
          when 'virtuoso'
            endpoint = LXDVirtuosoEndpoint.new("watdiv-#{size}-#{scheme}-virtuoso7-debian9")
          end
          watdiv_bench(endpoint, size, template, scheme, mode)
        end

        watdiv_bench_dependencies.append(task_dependency(task_name))
      end
    end
  end
end

# %w{10M}.each do |size|
#   %w{C3 L3 S2 S3 S5 S6}.each do |template|
#     engine = 'tripleprov'
#     scheme = 'namedgraphs'
#     mode = 'T'

#     task_name = "run_bench_#{engine}_#{size}_#{template}_#{scheme}_#{mode}"
#     desc "Run bench for #{engine} #{size} #{template} #{scheme} #{mode}"
#     named_task task_name do
#       tripleprov_bench(size, template)
#     end

#     watdiv_bench_dependencies.append(task_dependency(task_name))
#   end
# end

desc 'Run WatDiv bench'
task experiment_one_bench: watdiv_bench_dependencies
