# frozen_string_literal: true

def wikidata_bench(endpoint, template, size, mode, times = 5)
  puts "Starting workload #{[endpoint.name, template, size, mode].join('-')}"

  endpoint.start
  queries =
    Dir["wikidata/queries/#{template}/#{mode}/*.sparql"].sort

  results = "wikidata/results/#{endpoint.name}-#{template}-#{size}-#{mode}.csv"
  FileUtils.mkdir_p(File.dirname(results))

  # puts "Checking if this query produces timeouts"
  # out = endpoint.bench_query(queries.first)
  # puts "result time=#{out[0]} status=#{out[1]}"
  # if out[0] >= endpoint.timeout or out[1] != '200'
  #   puts "timeout detected"
  #   CSV.open(results, 'w') do |csv|
  #     csv << %w{engine template size mode query_id repetition time status}
  #     queries.each do |query|
  #       query_name = File.basename(query).sub(/.sparql$/, '')
  #       csv << [endpoint.name, template, size, mode, query_name, 1, endpoint.timeout, 500]
  #     end
  #   end
  #   endpoint.stop
  #   return
  # else
  #   puts "no timeout"
  # end

  queries.each do |query|
    puts "warming up #{query}"
    answers = query.gsub('/', '-')
                   .sub('.sparql', '.csv')
                   .sub('wikidata-queries-', "wikidata/answers/#{endpoint.name}-")
    FileUtils.mkdir_p(File.dirname(answers))
    File.open(answers, 'w') do |file|
      file.write endpoint.run_query(query)
    end
  end

  CSV.open(results, 'w') do |csv|
    csv << %w[engine template size mode query_id repetition time status]
    (1..times).each do |repetition|
      queries.each do |query|
        puts "running query #{query} (repetition #{repetition})"
        out = endpoint.bench_query(query)
        query_name = File.basename(query).sub(/.sparql$/, '')
        csv << [endpoint.name, template, size, mode, query_name, repetition, out[0], out[1]]
        csv.flush
      end
    end
  end

  endpoint.stop
end

wikidata_bench_dependencies = []

%w[fuseki virtuoso].each do |engine|
  %w[star minus union].each do |template|
    %w[B R P].each do |mode|
      size = '6'
      task_name = "wikidata_bench_#{engine}_#{template}_#{size}_#{mode}"

      desc "Run bench for engine=#{engine}, template=#{template}, size=#{size}, mode=#{mode}"
      named_task task_name do
        case engine
        when 'fuseki'
          endpoint = LXDFusekiEndpoint.new('wikidata-20200127-fuseki3-ubuntu2004')
        when 'virtuoso'
          endpoint = LXDVirtuosoEndpoint.new('wikidata-20200127-virtuoso7-debian9')
        end
        wikidata_bench(endpoint, template, size, mode)
      end

      wikidata_bench_dependencies.append(task_dependency(task_name))
    end
  end
end

desc 'Run bench'
task experiment_three_bench: wikidata_bench_dependencies
